<?php
use PrestaShop\PrestaShop\Core\Payment\PaymentOption;
class Borgunpayment extends PaymentModule{
	private $_html = '';
    private $_postErrors = array();

    public $extra_mail_vars;
	public $merchantid;    
    public $paymentgatewayid;
	public $merchantemailemail;
	public $mode;
	public $secretkey;
                    
    public function __construct()
    {
        $this->name = 'borgunpayment';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.2';
        $this->author = 'Muhammad Arif Amir';
        $this->controllers = array('payment', 'validation');
        $this->currencies = true;
        $this->currencies_mode = 'checkbox';

        $config = Configuration::getMultiple(array('MERCHANTID', 'PAYMENTGATEWAYID', 'MERCHANTEMAIL','SECRET'));
        if (!empty($config['MERCHANTID'])) {
            $this->merchantid = $config['MERCHANTID'];
        }
        if (!empty($config['PAYMENTGATEWAYID'])) {
            $this->paymentgatewayid = $config['PAYMENTGATEWAYID'];
        }
        if (!empty($config['MERCHANTEMAIL'])) {
            $this->merchantemail = $config['MERCHANTEMAIL'];
        }
		if (!empty($config['SECRET'])) {
            $this->secretkey = $config['SECRET'];
        }
		if (!empty($config['MODE'])) {
            $this->mode = $config['MODE'];
        }
        $this->bootstrap = true;
        
        parent::__construct();
        $this->displayName = $this->l('Borgun Payment Gateway');
        $this->description = $this->l('Accept payments for your products via Borgun Payment Gateway.');
        $this->confirmUninstall = $this->l('Are you sure about removing these details?');
        if (!isset($this->merchantid) || !isset($this->paymentgatewayid) || !isset($this->merchantemail) || !isset($this->secretkey) || !isset($this->mode)) {
            $this->warning = $this->l('Account details must be configured before using this module.');
        }
        if (!count(Currency::checkPaymentCurrencies($this->id))) {
            $this->warning = $this->l('No currency has been set for this module.');
        }
        $this->extra_mail_vars = array(
            '{borgunpayment_merchantid}' => Configuration::get('MERCHANTID'),
            '{borgunpayment_paymentgatewayid}' => nl2br(Configuration::get('PAYMENTGATEWAYID')),
            '{borgunpayment_merchantemail}' => nl2br(Configuration::get('MERCHANTEMAIL')),
            '{borgunpayment_secretkey}' => nl2br(Configuration::get('SECRET')),
            '{borgunpayment_mode}' => nl2br(Configuration::get('MODE'))
        );
    }
    
    public function install() {
        if (!parent::install() || !$this->registerHook('paymentOptions') || !$this->registerHook('paymentReturn')) {
            return false;
        }
        return true;
    }

    public function uninstall() {
        if (!Configuration::deleteByName('MERCHANTID') || !Configuration::deleteByName('PAYMENTGATEWAYID') || !Configuration::deleteByName('MERCHANTEMAIL') || !Configuration::deleteByName('SECRET') || !Configuration::deleteByName('MODE') || !parent::uninstall()) {
            return false;
        }
        return true;
    }
    
    private function _postValidation() {
        if (Tools::isSubmit('btnSubmit')) {
            if (!Tools::getValue('MERCHANTID'))
                $this->_postErrors[] = $this->l('Merchant ID is required.');
            elseif (!Tools::getValue('PAYMENTGATEWAYID'))
                $this->_postErrors[] = $this->l('Payment gateway id  is required.');
            elseif (!Tools::getValue('MERCHANTEMAIL'))
                $this->_postErrors[] = $this->l('Merchant email is required.');
			elseif (!Tools::getValue('SECRET'))
				$this->_postErrors[] = $this->l('Secret Key is required.');
			elseif (!Tools::getValue('MODE'))
				$this->_postErrors[] = $this->l('Mode is required.');
        }
    }

    private function _postProcess() {
        if (Tools::isSubmit('btnSubmit')) {
            Configuration::updateValue('MERCHANTID', Tools::getValue('MERCHANTID'));
            Configuration::updateValue('PAYMENTGATEWAYID', Tools::getValue('PAYMENTGATEWAYID'));
            Configuration::updateValue('MERCHANTEMAIL', Tools::getValue('MERCHANTEMAIL'));
            Configuration::updateValue('SECRET', Tools::getValue('SECRET'));
            Configuration::updateValue('MODE', Tools::getValue('MODE'));
        }
        $this->_html .= $this->displayConfirmation($this->l('Settings updated'));
    }

    private function _displayBankWire() {
        return $this->display(__FILE__, 'infos.tpl');
    }
    
    public function getContent() {
        if (Tools::isSubmit('btnSubmit')) {
            $this->_postValidation();
            if (!count($this->_postErrors)) {
                $this->_postProcess();
            } else {
                foreach ($this->_postErrors as $err) {
                    $this->_html .= $this->displayError($err);
                }
            }
        } else
            $this->_html .= '<br />';

        $this->_html .= $this->_displayBankWire();
        $this->_html .= $this->renderForm();

        return $this->_html;
    }

    public function hookPaymentOptions($params) {

        if (!$this->active)
            return;
        if (!$this->checkCurrency($params['cart']))
            return;
			
		/* Lets start playing with the borgun form */
		
		$cart = $this->context->cart;
		$cart_details = $cart->getSummaryDetails(null, true);	
		
		if ((int)Configuration::get('MODE') == 1)
			// Sandbox url
			$action_url = 'https://test.borgun.is/SecurePay/default.aspx';
		else
			$action_url = 'https://securepay.borgun.is/securepay/default.aspx';

		$shop_url           = Borgunpayment::getShopDomainSsl(true, true);
		$bsecretkey			= Configuration::get('SECRET');
		$bmerchantemail		= Configuration::get('MERCHANTEMAIL');
		$bpaymentgatewayid	= Configuration::get('PAYMENTGATEWAYID');
		$bmerchantid		= Configuration::get('MERCHANTID');
		$success_url		= $this->context->link->getModuleLink('borgunpayment', 'payment');
		//$hash				= md5($bmerchantid.$success_url.$bsecretkey);
		
		// Calculate the cart total
		$Total = 0;
		$only_discounts   = $cart->getOrderTotal(true, Cart::ONLY_DISCOUNTS);
		$products         = $cart->getProducts();
		$only_shipping    = $cart->getOrderTotal(true, Cart::ONLY_SHIPPING);
		$total_tax        = (float)$this->context->cart->getOrderTotal(true) - (float)$this->context->cart->getOrderTotal(false);
		
		if( $products ){
			foreach( $products as $borgun_product ){
				
				$Total = $Total+ ($borgun_product['quantity'] * round($borgun_product['price'],2));
			}
		}
		
		if ( $only_shipping ){
			
			$Total = $Total+ round($only_shipping,2);
		}
		
		if ( $total_tax ){
			
			$Total = $Total+ round($total_tax,2);
		}
		
		$currency = new Currency((int)$cart->id_currency);
		$currency_code = $currency->iso_code;
		$order_id	= $cart->id;
		
		// Calculation of hash using hmac
		$hash = $bmerchantid.'|'.$success_url.'|'.$success_url.'|'.$order_id.'|'.$Total.'|'.$currency_code;
		$hash = utf8_encode(trim($hash));		
		$hash = hash_hmac('sha256', $hash,$bsecretkey);
		
		
		$billing_address          = new Address((int)$this->context->cart->id_address_invoice);
		$billing_address->country = new Country((int)$billing_address->id_country);
		$billing_address->state   = new State((int)$billing_address->id_state);
		
		$fname        = $billing_address->firstname;	
		$lname        = $billing_address->lastname;
		$full_name    = $fname.' '.$lname;

        $this->smarty->assign(array(
			'action'            => $action_url,
			'pagetype'          => '0',
            'skipreceiptpage'   => '0',
			'currency'          => new Currency((int)$cart->id_currency),
			'returnurlcancel'   => $this->context->link->getPageLink('order.php'),
			'returnurlerror'    => $this->context->link->getModuleLink('borgunpayment', 'payment'),
            'merchantemail'	    => $bmerchantemail,
			'returnurlsuccess'  => $success_url,
			'language'          => 'en',
			'checkhash'         => $hash,
			'merchantid'        => $bmerchantid,
            'paymentgatewayid'  => $bpaymentgatewayid,
			'orderid'           => $cart->id,
			'cart'				=> $cart,
			'buyername'			=> $full_name,
			'total_tax'			=>(float)$this->context->cart->getOrderTotal(true) - (float)$this->context->cart->getOrderTotal(false),
			'this_path'         => $this->_path,
            'this_path_borgun'  => $this->_path,
            'this_path_bw'      => $this->_path,
            'this_path_ssl'     => Configuration::get('PS_FO_PROTOCOL').$_SERVER['HTTP_HOST']. __PS_BASE_URI__ . 'modules/' . $this->name . '/'
        ));
		
		$newOption = new PaymentOption();
        $newOption->setModuleName($this->name)
            ->setCallToActionText($this->trans('Borgun Payment'))
            ->setAction($action_url)
            ->setBinary(true)
            ->setAdditionalInformation($this->fetch('module:borgunpayment/views/templates/hook/payment.tpl'))
            ->setLogo(Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/borgunlogo.png')
        );
        $payment_options = [
            $newOption,
        ];

        return $payment_options;
    }
	
	public static function getShopDomainSsl($http = false, $entities = false)
	{
		if (method_exists('Tools', 'getShopDomainSsl'))
			return Tools::getShopDomainSsl($http, $entities);
		else
		{
			if (!($domain = Configuration::get('PS_SHOP_DOMAIN_SSL')))
				$domain = self::getHttpHost();
			if ($entities)
				$domain = htmlspecialchars($domain, ENT_COMPAT, 'UTF-8');
			if ($http)
				$domain = (Configuration::get('PS_SSL_ENABLED') ? 'https://' : 'http://').$domain;
			return $domain;
		}
	}
	
    public function hookPaymentReturn($params) {
        
		if (!$this->active)
            return;

        if (!isset($params['order']) || ($params['order']->module != $this->name))
            return false;
        
        if (isset($params['order']) && Validate::isLoadedObject($params['order']) && isset($params['order']->valid)) {
            $this->smarty->assign(array(
                'id_order' => $params['order']->id,
                'valid' => $params['order']->valid,
                ));
        }
        if (isset($params['order']->reference) && !empty($params['order']->reference)) {
            $this->smarty->assign('reference', $params['order']->reference);
        }
        $this->smarty->assign(array(
            'shop_name' => $this->context->shop->name,
            'reference' => $params['order']->reference,
            'contact_url' => $this->context->link->getPageLink('contact', true)
            ));
        return $his->fetch('mymodule/views/templates/hook/payment_return.tpl'); 
    }

    public function checkCurrency($cart) {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    public function renderForm() {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Borgun Configuration'),
                    'icon' => 'icon-envelope'
                ),
				'input' => array(
					array(
						'type' => 'select',
						'lang' => true,
						'label' => $this->l('Mode'),
						'name' => 'MODE',
						'options' => array(
							'query' => array(
							  array(
								'id_option' => 1, 
								'name' => 'Sandbox' 
							  ),
							  array(
								'id_option' => 2,
								'name' => 'Live'
							  ),
							),
						'id' => 'id_option', 
						'name' => 'name'
						),
					),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Borgun Merchant ID'),
                        'name' => 'MERCHANTID'
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Borgun Payment Gateway ID'),
                        'name' => 'PAYMENTGATEWAYID'
                    ),
					array(
                        'type' => 'text',
                        'label' => $this->l('Borgun Secret Key'),
                        'name' => 'SECRET'
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Merchant Email'),
                        'name' => 'MERCHANTEMAIL'
                    ),
					
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                )
            ),
        );

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();
        $helper->id = (int) Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'btnSubmit';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );

        return $helper->generateForm(array($fields_form));
    }

    public function getConfigFieldsValues() {
        return array(
            'MERCHANTID' => Tools::getValue('MERCHANTID', Configuration::get('MERCHANTID')),
            'PAYMENTGATEWAYID' => Tools::getValue('PAYMENTGATEWAYID', Configuration::get('PAYMENTGATEWAYID')),
            'MERCHANTEMAIL' => Tools::getValue('MERCHANTEMAIL', Configuration::get('MERCHANTEMAIL')),
            'SECRET' => Tools::getValue('SECRET', Configuration::get('SECRET')),
            'MODE' => Tools::getValue('MODE', Configuration::get('MODE')),
        );
    }

}