<?php
/*
* 2007-2014 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2014 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

/**
 * @since 1.5.0
 */

class BorgunpaymentPaymentModuleFrontController extends ModuleFrontController
{
	public $ssl = true;
	public $display_column_left = false;
	public $display_column_right = false;

	/**
	 * @see FrontController::initContent()
	 */
	public function initContent()
	{
		parent::initContent();

		$cart = $this->context->cart;
		
		if (!$this->module->checkCurrency($cart))
			Tools::redirect('index.php?controller=order');
			
		$orderid = Tools::getValue('orderid');
		$orderhash = Tools::getValue('orderhash');
		$step = Tools::getValue('step');
		$status = Tools::getValue('status');
		$customField = Tools::getValue('customField');
		$currency = Tools::getValue('currency');
		
		if($status=='OK'){
			if(!empty($orderhash)){
				$secretkey 				= Configuration::get('SECRET');
				$amount					= $customField;
				//$checkhash 				= md5($orderid.$amount.$secretkey);
				
				// Calculate the hash using hmac
				$OrderHashMessage = $orderid.'|'.$amount.'|'.$currency;
				$OrderHashMessage = utf8_encode(trim($OrderHashMessage));		
				$checkhash = hash_hmac('sha256', $OrderHashMessage,$secretkey);
				
				
				if($orderhash==$checkhash){
					//Update the order if hash matches and redirec the user to success template.
					if ($step == 'Confirmation')
						$order_status = (int)Configuration::get('PS_OS_PAYMENT');
					
					$customer = new Customer($cart->id_customer);
					if (!Validate::isLoadedObject($customer))
						Tools::redirect('index.php?controller=order&step=1');

					$currency = $this->context->currency;
					$total = (float)$cart->getOrderTotal(true, Cart::BOTH);
					$mailVars = array(
						'{borgunpayment_merchantemailid}' => Configuration::get('MERCHANTID'),
						'{borgunpayment_paymentgatewayid}' => nl2br(Configuration::get('PAYMENTGATEWAYID')),
						'{borgunpayment_merchantemail}' => nl2br(Configuration::get('MERCHANTEMAIL')),
						'{borgunpayment_secretkey}' => nl2br(Configuration::get('SECRET')),
						'{borgunpayment_mode}' => nl2br(Configuration::get('MODE'))
					);

					$this->module->validateOrder($cart->id, $order_status, $total, $this->module->displayName, NULL, $mailVars, (int)$currency->id, false, $customer->secure_key);
					Tools::redirect('index.php?controller=order-confirmation&id_cart='.$cart->id.'&id_module='.$this->module->id.'&id_order='.$this->module->currentOrder.'&key='.$customer->secure_key);
				}else{
					$this->context->link->getPageLink('order.php');	
				}
			}
		}
		
		if($status=='ERROR'){

			/*$this->context->smarty->assign(array(
				'error_description' => Tools::getValue('errordetail'),
			));
			$this->setTemplate('error.tpl');
			*/
			Tools::redirect($this->context->link->getPageLink('order.php'));
		}		
	}
}
